"""

Generates appropriate replacements (masses, BC conditions, and abundances for MESA
inlist files. Outputs lists of replacements that are inputs to make_replacements.py

Returns:
    the list of replacements
    
See Also:
    make_replacements: takes the list of replacements as an input
    
Example:
    >>> replist = make_inlist_inputs(runname, 'VeryLow')
    
Acknowledgment:
    Thanks to Joshua Burkart for providing assistance with and content for earlier versions of this code.
    
"""

import sys
import numpy as np
    
def make_inlist_inputs(runname): 
    
    #Array of all masses

    metalgrid = np.array([0.02,0.002])
    periodgrid = np.array([0.25,0.5,0.75,1.0,1.25,1.5,2.0,99.])

    #Create mass lists
    mapfunc = lambda var: np.str(int(var)) if var == int(var) else np.str(var)
    metallist = map(mapfunc, metalgrid) #bigmassgrid[massindex]
    periodlist = map(mapfunc, periodgrid) #bigmassgrid[massindex]


    #Make list of [replacement string, values]
    replist = [\
            ["<<metal>>", metallist],\
	    ["<<period>>", periodlist],\
        ]

    #print(replist)

    return replist
