#!/usr/bin/env python 

"""

Calls all of the necessary routines to submit a grid of specified name and metallicity.

Args:
    The name of the grid
    The name of the initial model
    Companion mass

Example:
    #For new format
    >>> ./submit_jobs.py 30grid 10_from_30.mod 9.7

"""

import os
import sys
import shutil

from scripts import make_slurm_sh
from scripts import make_inlist_inputs
from scripts import make_replacements

if __name__ == "__main__":

    if len(sys.argv) == 4:
        runname = sys.argv[1]
        modname = sys.argv[2]
        M2 = float(sys.argv[3])
    elif len(sys.argv) < 2:
        print "Usage: ./submit_jobs name_of_grid Mass"
        sys.exit(0)


    dirname = os.path.join(os.environ['GRID_DIR'], runname) #set in .bashrc  
    
    #Create a working directory
    try:
        os.mkdir(dirname)
    except OSError:
        print "The directory already exists."
        sys.exit(0)
    
    #Generate inlists using template inlist files
        
    #Replace inlist1    
    tempstor_inlist_dir = os.path.join(os.environ['MESAWORK_DIR'], 'inlists/inlists_'+'_'.join(runname.split('/')))
    new_inlist_name = 'p<<period>>z<<metal>>.inlist1'
    path_to_inlist = os.path.join(os.environ['CODE_DIR'],'mesafiles/inlist1')
    make_replacements.make_replacements(make_inlist_inputs.make_inlist_inputs(runname), modname, M2,\
    new_inlist_name, direc=tempstor_inlist_dir, file_base=path_to_inlist)
    
    #Replace inlist_project
    tempstor_inlist_dir_p = os.path.join(os.environ['MESAWORK_DIR'], 'inlists/inlists_project_'+'_'.join(runname.split('/')))
    new_inlist_name = 'p<<period>>z<<metal>>.inlist1_project'
    path_to_inlist = os.path.join(os.environ['CODE_DIR'],'mesafiles/inlist_project')
    make_replacements.make_replacements(make_inlist_inputs.make_inlist_inputs(runname), modname, M2,\
    new_inlist_name, direc=tempstor_inlist_dir_p, file_base=path_to_inlist)
        
    inlist_list = os.listdir(tempstor_inlist_dir)
    inlist_list.sort()


    for inlistname in inlist_list:
        print(inlistname)
        #Make individual directories for each mass
        onemassdir = inlistname.replace('.inlist1', '_dir')

        #Define an absolute path to this directory.
        path_to_onemassdir = os.path.join(dirname, onemassdir)

        #Copy over the contents of the template directory
        shutil.copytree(os.path.join(os.environ['MESAWORK_DIR'], "cleanworkdir"), path_to_onemassdir)

        #Populate each directory with appropriate inlists and rename as inlist1
        shutil.copy(os.path.join(tempstor_inlist_dir,inlistname), os.path.join(path_to_onemassdir, 'inlist1'))
        
        #Populate each directory with appropriate inlist_project
        inlistpname = inlistname+'_project'
        shutil.copy(os.path.join(tempstor_inlist_dir_p,inlistpname), os.path.join(path_to_onemassdir, 'inlist_project'))

        #Copy He star model into directory
        shutil.copy(os.path.join(os.environ['MESAWORK_DIR'], 'hemodels/', modname),\
                os.path.join(path_to_onemassdir, modname))
        
        #Populate each directory with the most recent my_history_columns.list and run_star_extras.f
        #shutil.copy(os.path.join(os.environ['CODE_DIR'], 'mesafiles/inlist_project'),\
        #        os.path.join(path_to_onemassdir, 'inlist_project'))
        shutil.copy(os.path.join(os.environ['CODE_DIR'], 'mesafiles/history_columns.list'),\
                os.path.join(path_to_onemassdir, 'history_columns.list'))
        shutil.copy(os.path.join(os.environ['CODE_DIR'], 'mesafiles/profile_columns.list'),\
                os.path.join(path_to_onemassdir, 'profile_columns.list'))
        shutil.copy(os.path.join(os.environ['CODE_DIR'], 'mesafiles/run_star_extras.f'),\
                os.path.join(path_to_onemassdir, 'src/run_star_extras.f'))

        #Create and move the SLURM file to the correct directory
        runbasefile = os.path.join(os.environ['CODE_DIR'], 'mesafiles/SLURM_grid.sh')
        slurmfile = make_slurm_sh.make_slurm_sh(inlistname, path_to_onemassdir, runbasefile)
        shutil.move(slurmfile, path_to_onemassdir)
        
        #cd into the individual directory and submit the job
        os.chdir(path_to_onemassdir)
        #print "sbatch " + slurmfile
        ##for testing, turn off
        os.system("sbatch "+slurmfile) 
        os.chdir(os.environ['CODE_DIR'])
